package ru.ulstu.report.controller;

import org.springframework.data.domain.Page;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import ru.ulstu.aspirant.service.AspirantService;
import ru.ulstu.indicator.model.Indicator;
import ru.ulstu.model.OffsetablePageRequest;
import ru.ulstu.report.model.Report;
import ru.ulstu.report.model.ReportListForm;
import ru.ulstu.report.model.ReportPeriod;
import ru.ulstu.report.service.ReportPeriodService;
import ru.ulstu.report.service.ReportService;

import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

@Controller
@RequestMapping("report")
public class ReportController {
    private static final Integer DEFAULT_PAGE_SIZE = 1;
    private static final Integer DEFAULT_PAGE_NUMBER = 1;

    private final AspirantService aspirantService;
    private final ReportService reportService;
    private final ReportPeriodService reportPeriodService;

    public ReportController(AspirantService aspirantService,
                            ReportService reportService,
                            ReportPeriodService reportPeriodService) {
        this.aspirantService = aspirantService;
        this.reportService = reportService;
        this.reportPeriodService = reportPeriodService;
    }

    @GetMapping("reportList")
    public String getReportPeriods(Model model) {
        model.addAttribute("reportListForm", new ReportListForm());
        List<ReportPeriod> periods = reportPeriodService.getReportPeriods();
        model.addAttribute("reportPeriods", periods);
        model.addAttribute("canCreate", !periods.isEmpty() && reportService.canCreateReport(periods.getFirst()));
        return "report/reportList";
    }

    @PostMapping("reportList")
    public String getReportPeriods(ReportListForm reportListForm, Model model) {
        model.addAttribute("reportListForm", reportListForm);
        model.addAttribute("reports", reportService.getReports(reportListForm.getReportPeriod()));
        List<ReportPeriod> periods = reportPeriodService.getReportPeriods();
        model.addAttribute("reportPeriods", periods);
        model.addAttribute("canCreate", !periods.isEmpty() && reportService.canCreateReport(periods.getFirst()));
        return "report/reportList";
    }

    @GetMapping("editReport/{reportId}")
    public String createReport(@PathVariable("reportId") Integer reportId,
                               Model model,
                               @RequestParam Optional<Integer> page,
                               @RequestParam Optional<Integer> size) {
        int currentPage = page.orElse(DEFAULT_PAGE_NUMBER);
        int pageSize = size.orElse(DEFAULT_PAGE_SIZE);

        Page<Indicator> indicators = aspirantService.getIndicatorsByCourse(new OffsetablePageRequest(currentPage - 1, pageSize));
        model.addAttribute("indicators", indicators);
        int totalPages = indicators.getTotalPages();
        if (totalPages > 0) {
            List<Integer> pageNumbers = IntStream.rangeClosed(1, totalPages)
                    .boxed()
                    .collect(Collectors.toList());
            model.addAttribute("pageNumbers", pageNumbers);
        }

        model.addAttribute("report", new Report(0));
        return "report/editReport";
    }
}
